module RedmineCrm
  module Liquid
    module Filters
      # include ApplicationHelper
      # include Rails.application.routes.url_helpers

      def textilize(input)
        RedCloth3.new(input).to_html
      end

      # example:
      #   {{ today | plus_days: 2 }}
      def plus_days(input, distanse)
        return '' if input.nil?
        days = distanse.to_i
        input.to_date + days.days rescue 'Invalid date'
      end

      # example:
      #   {{ today | date_range: '2015-12-12' }}
      def date_range(input, distanse)
        return '' if input.nil?
        (input.to_date - distanse.to_date).to_i rescue 'Invalid date'
      end

      # example:
      #   {{ now | utc }}
      def utc(input)
        return '' if input.nil?
        input.to_time.utc rescue 'Invalid date'
      end

      def default(input, value)
        input.blank? ? value : input
      end

      def underscore(input)
        input.to_s.gsub(' ', '_').gsub('/', '_').underscore
      end

      def dasherize(input)
        input.to_s.gsub(' ', '-').gsub('/', '-').dasherize
      end

      def shuffle(array)
        array.to_a.shuffle
      end

      def random(input)
        rand(input.to_i)
      end

      # example:
      #   {{ "http:://www.example.com?key=hello world" | encode }}
      #
      #   => http%3A%3A%2F%2Fwww.example.com%3Fkey%3Dhello+world
      def encode(input)
        Rack::Utils.escape(input)
      end

    protected

      # Convert an array of properties ('key:value') into a hash
      # Ex: ['width:50', 'height:100'] => { :width => '50', :height => '100' }
      def args_to_options(*args)
        options = {}
        args.flatten.each do |a|
          if a =~ /^(.*):(.*)$/
            options[$1.to_sym] = $2
          end
        end
        options
      end

      # Write options (Hash) into a string according to the following pattern:
      # <key1>="<value1>", <key2>="<value2", ...etc
      def inline_options(options = {})
        return '' if options.empty?
        (options.stringify_keys.sort.to_a.collect { |a, b| "#{a}=\"#{b}\"" }).join(' ') << ' '
      end
    end
  end

  ::Liquid::Template.register_filter(RedmineCrm::Liquid::Filters)
end
